""" Loop unrolling transformation in ufl

NB: Doing this in UFL is not the best level of abstractions to do this.
    However, for the sum factorization code paths, there are some advantages
    in traversing unrolled expression trees.
"""

from ufl.algorithms import MultiFunction
from dune.perftool.ufl.transformations import ufl_transformation
from dune.perftool.ufl.flatoperators import construct_binary_operator
from dune.perftool.options import get_option

import ufl.classes as uc


class UnrollDimensionLoops(MultiFunction):
    def __init__(self):
        self.replace = {}
        self._indices_backup = []
        self.indices = None
        MultiFunction.__init__(self)

    call = MultiFunction.__call__

    def expr(self, o):
        return self.reuse_if_untouched(o, *tuple(self(op) for op in o.ufl_operands))

    def indexed(self, o):
        self._indices_backup.append(self.indices)
        self.indices = self.call(o.ufl_operands[1])

        ret = self.call(o.ufl_operands[0])

        if self.indices is not None:
            ret = uc.Indexed(ret, self.indices)

        self.indices = self._indices_backup.pop()

        return ret

    def multi_index(self, o):
        return uc.MultiIndex(tuple(self.replace.get(i, i) for i in o))

    def index_sum(self, o):
        operands = []

        for i in range(o.dimension()):
            self.replace[o.ufl_operands[1][0]] = uc.FixedIndex(i)
            operands.append(self.call(o.ufl_operands[0]))
            del self.replace[o.ufl_operands[1][0]]

        return construct_binary_operator(tuple(operands), uc.Sum)

    def list_tensor(self, o):
        index = self.indices[0]
        index = index._value
        self.indices = self.indices[1:]
        if len(self.indices) == 0:
            self.indices = None
        return self.call(o.ufl_operands[index])

    def component_tensor(self, o):
        assert len(self.indices) == len(o.ufl_operands[1])

        # Update the index mapping
        for i, ind in enumerate(o.ufl_operands[1]):
            self.replace[ind] = self.indices[i]

        self.indices = None
        ret = self.call(o.ufl_operands[0])

        for i, ind in enumerate(o.ufl_operands[1]):
            del self.replace[ind]

        return ret


@ufl_transformation(name="unroll")
def unroll_dimension_loops(expr):
    return UnrollDimensionLoops()(expr)
#     if get_option("unroll_dimension_loops"):
#         return UnrollDimensionLoops()(expr)
#     else:
#         return expr
